/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

// @(#) $Id: ITabularDomData.java 2089 2007-11-28 13:56:13Z s3460 $
package at.spardat.xma.datasource;

/**
 * An <tt>ITabularDomData</tt> is a special kind of tabular data, collectively known
 * under the german word <tt>Domne</tt> at SPARDAT. <tt>ITabularDomData</tt>s are
 * tables whose structure is fixed to the following columns:
 * <ol>
 * <li><tt>COD_KEY</tt>. Is of type String. Identifies a row to the programmer. This column acts
 * as key. No two rows may have the same key. This column is mandatory, i.e.,
 * empty values are not allowed.
 * <li><tt>SHORT_VALUE</tt>. Of type String. Short description of the domain row for the
 * end user. This column usually holds a localized String. A cell of this
 * column may be empty. 
 * <li><tt>LONG_VALUE</tt>. Of type String. More verbose description of the domain row for
 * end user purpose. This column usually holds a localized String. A cell
 * of this column may be empty.
 * <li><tt>VALID_FROM</tt>. Of type Date. Holds the first day this row
 * is visible to the end user. Cells may be empty meaning that the visibility
 * of the row starts at negative inifinite time.
 * <li><tt>VALID_TO</tt>. Of type Date. Holds the last day this row is
 * visible to the end user. Emptyness of this columns cells means that
 * there is no time limit of visibility.
 * </ol>
 * 
 * @author YSD, 20.06.2003 18:49:09
 */
public interface ITabularDomData extends ITabularData {

    /**
     * Returns a domain row at a provided index.
     * 
     * @param index zero based index, may range from zero up to <tt>size()-1</tt>.
     * @return the <tt>IDomRow</tt> object at the requested index.
     * @exception IllegalArgumentException if index is out of range.
     */
    public IDomRow getDomRow (int index);
    
    /**
     * Returns a row for a provided <tt>key</tt> or <tt>null</tt>
     * if there is no row with that key. All rows are searched, 
     * regardless of their valid-time-range.
     * 
     * @param key the <tt>COD_KEY</tt> of the searched row.
     * @return <tt>IDomRow</tt> object or <tt>null</tt>
     */
    public IDomRow getDomRow (String key);
}
