/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

//@(#) $Id: TaggedAsciiDeserializer.java 2089 2007-11-28 13:56:13Z s3460 $
package at.spardat.xma.serializer;

import java.io.IOException;

import at.spardat.enterprise.exc.SysException;

/**
 * Is able to read the byte stream written by <tt>TaggedAsciiSerializer</tt>.
 * 
 * @author MS, Feb 12, 2005 9:24:13 AM
 */
public class TaggedAsciiDeserializer extends AsciiDeserializer {

    /**
     * Constructor
     * 
     * @param data    serialized array of data 
     * @param length  first <tt>length</tt> bytes in <tt>data</tt> are used.
     */
    public TaggedAsciiDeserializer (byte[] data, int length) {
        super (data, length);
    }
    
    /**
     * Constructor 
     * 
     * @param data  serialized array of data
     */
    public TaggedAsciiDeserializer (byte [] data) {
        super (data);
    }
    
    public TaggedAsciiDeserializer (byte [] data, int offset, int length) {
        super (data, offset, length);
    }
    
    /**
     * There must be three components read: First, a label, followed by '=', 
     * then the value.
     * 
     * @see at.spardat.xma.serializer.AsciiDeserializer#readStringEscaped()
     */
    protected String readStringEscaped() {
        String label = super.readStringEscaped();
        if (super.fArr.getRel(-1) != '=') throw new SysException ("missing = in stream");
        return super.readStringEscaped();
    }
    
    /**
     * @see at.spardat.xma.serializer.AsciiDeserializer#readSerializedBytes()
     */
    public byte[] readSerializedBytes() throws IOException {
        String label = super.readStringEscaped();
        if (super.fArr.getRel(-1) != '=') throw new SysException ("missing = in stream");
        return super.readSerializedBytes();
    }

    
}
